(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: image.mli,v 1.51 2013-10-07 14:44:53 fclement Exp $ *)

open Segmentation_types;;

(*
val gimage_digest : ('a, 'a gimage) Utype_digest.lifter;;
val cimage_digest : ('a, 'b, ('a, 'b) cimage) Utype_digest.lifter2;;
*)

val same_gimage : 'a gimage -> 'a gimage -> bool;;
val same_cimage : ('a, 'b) cimage -> ('a, 'b) cimage -> bool;;

val last_zone_number : ('a, 'b) pimage -> zone_number;;

(** {6 Converters} *)

(*val image_combine : image Tuple.Triple.t -> image;;*)

val scalar_to_channels :
  ('a -> int) -> ((color -> color) -> 'a -> color) Tuple.Triple.t
;;

val gimage_of_image : (color -> 'a) -> image -> 'a gimage;;
val image_of_full_gimage : ('a -> color) -> 'a gimage -> image;;

val set_pixmap_of_full_gimage :
  (color -> int -> int -> unit) -> ('a -> color) -> 'a gimage -> unit
;;

val constant_zone :
  ('a gimage -> 'a) -> 'a gimage ->
  ('a gimage -> 'a gimage -> 'a cost) -> 'a gimage ->
  'a zc_gimage
;;

val cimage_of_gimage :
  ('b -> zone_number -> 'a zc_gimage -> 'a best_zone_cutting) ->
  ('a gimage -> 'a) -> ('a gimage -> 'a gimage -> 'a cost) ->
  (('a, 'b) pimage -> 'a cost) ->
  'a gimage -> ('a, 'b) cimage
;;
val gimage_of_cimage : ('a, 'b) cimage -> 'a gimage;;

val sub_gimage : ('a -> 'a -> 'a) -> 'a gimage -> 'a gimage -> 'a gimage;;
(** [sub_gimage f gim sgim] returns the gimage with the domain of [sgim], but
 with images obtained by the application of [f] to the images of [gim] and
 [sgim].
 [gim] is supposed to be a full gimage, and the domain of [sgim] is supposed
 to be a subset of the domain of [gim].
 For instance, the partial application [sub_gimage Scalar.( -. ) gdata]
 computes the gradient in each zone value. *)

val param : 'a zc_gimage -> 'a gimage;;

(** {6 Iterators} *)

val gimage_map : ('a -> 'b) -> 'a gimage -> 'b gimage;;

val pimage_iter : (zone_number -> 'a zone -> unit) -> ('a, 'b) pimage -> unit;;

val pimage_fold_left :
  ('accu -> zone_number -> 'a zone -> 'accu) ->
  'accu -> ('a, 'b) pimage -> 'accu
;;

val pimage_iter_to_do :
  (zone_number -> 'a zc_gimage -> unit) -> ('a, 'b) pimage -> unit
;;
val pimage_iter_value :
  (zone_number -> 'a zc_gimage -> unit) -> ('a, 'b) pimage -> unit
;;
val pimage_fold_left_value :
  ('accu -> zone_number -> 'a zc_gimage -> 'accu) ->
  'accu -> ('a, 'b) pimage -> 'accu
;;

val bounds_of_domain : Umatrix_nonempty_set.t -> bounds;;
(** Domain operations. *)
