(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: utype_digest.mli,v 1.9 2014/01/10 15:28:04 fclement Exp $ *)

(** {3 Runtime identification of types} *)

type 'a t = Digest.t;;
(** The type of MD5 digests for type expressions: 16-character strings. *)

type ('a, 'b) lifter = 'a t -> 'b t;;
type ('a, 'b, 'c) lifter2 = 'a t -> 'b t -> 'c t;;
type ('a, 'b, 'c, 'd) lifter3 = 'a t -> 'b t -> 'c t -> 'd t;;
type ('a, 'b, 'c, 'd, 'e) lifter4 = 'a t -> 'b t -> 'c t -> 'd t -> 'e t;;
type ('a, 'b, 'c, 'd, 'e, 'f, 'g) lifter6 =
  'a t -> 'b t -> 'c t -> 'd t -> 'e t -> 'f t -> 'g t
;;

type type_name = string;;

val exception_digest : exn t;;
val unit_digest : unit t;;
val bool_digest : bool t;;
val char_digest : char t;;
val int_digest : int t;;
val int32_digest : int32 t;;
val int64_digest : int64 t;;
val nativeint_digest : nativeint t;;
val float_digest : float t;;
val string_digest : string t;;
val option_digest : ('a, 'a option) lifter;;
val list_digest : ('a, 'a list) lifter;;
val array_digest : ('a, 'a array) lifter;;
val couple_digest : ('a, 'b, 'a * 'b) lifter2;;
val triple_digest : ('a, 'b, 'c, 'a * 'b * 'c) lifter3;;
val arrow_digest : ('a, 'b, 'a -> 'b) lifter2;;
(** The runtime identification of builtin types. *)

val fpclass_digest : Pervasives.fpclass t;;
val in_channel_digest : Pervasives.in_channel t;;
val out_channel_digest : Pervasives.out_channel t;;
val open_flag_digest : Pervasives.open_flag t;;
val ref_digest : ('a, 'a ref) lifter;;
val format6_digest : ('a, 'b, 'c, 'd, 'e, 'f, 'g) lifter6;;
val format4_digest : ('a, 'b, 'c, 'd, 'e) lifter4;;
val format_digest : ('a, 'b, 'c, 'd) lifter3;;
(** The runtime identification of types from Pervasives. *)

val list1_digest : ('a, 'a Ulist1.t) lifter;;
(** The runtime identification of types from Ulib modules. *)

val of_string : string -> Digest.t;;

val gen_digest : Digest.t -> type_name -> 'a t;;
val gen_unary_digest : Digest.t -> type_name -> ('a, 'b) lifter;;
val gen_binary_digest : Digest.t -> type_name -> ('a, 'b, 'c) lifter2;;
val gen_ternary_digest : Digest.t -> type_name -> ('a, 'b, 'c, 'd) lifter3;;
val gen_succ_digest :
  (Digest.t -> type_name -> 'a) -> Digest.t -> type_name -> 'b t -> 'a
;;
(** Generators of runtime identification for complex types. *)
